from Components.MenuList import MenuList
from Components.ActionMap import ActionMap
from Components.Label import Label
from Components.MultiContent import MultiContentEntryText, MultiContentEntryPixmapAlphaTest
from Tools.NumericalTextInput import NumericalTextInput
from Screens.NumericalTextInputHelpDialog import NumericalTextInputHelpDialog

from Components.config import ConfigText
from Components.config import ConfigSubsection
from Components.config import config

from enigma import eListboxPythonMultiContent, eListbox, gFont, RT_HALIGN_LEFT, RT_VALIGN_TOP, RT_WRAP

from Screens.InputBox import InputBox
from Screens.Screen import Screen

from Tools.LoadPixmap import LoadPixmap
from Tools.Directories import resolveFilename, SCOPE_PLUGINS, SCOPE_SKIN_IMAGE

from twisted.web.client import downloadPage

from Plugins.Extensions.VlcPlayer.VlcServerConfig import vlcServerConfig
from Plugins.Extensions.VlcPlayer.VlcServer import VlcServer
from Plugins.Extensions.VlcPlayer.VlcServerList import VlcServerListScreen

import gdata.service
import urllib
import httplib

import re
import os


def YouTubeEntryComponent(youTubeEntry):
	res = [ youTubeEntry ]

	res.append(MultiContentEntryText(pos = (150, 5), size = (385, 45), font = 0, flags = RT_HALIGN_LEFT | RT_VALIGN_TOP| RT_WRAP, text = youTubeEntry.title))
	res.append(MultiContentEntryText(pos = (150, 46), size = (385, 56), font = 1, color = 0xFFA323, color_sel = 0xFFA323, flags = RT_HALIGN_LEFT | RT_VALIGN_TOP| RT_WRAP, text = youTubeEntry.description))

	if youTubeEntry.thumbnail is None:
		png = LoadPixmap(resolveFilename(SCOPE_SKIN_IMAGE, "skin_default/icons/plugin.png"))
	else:
		png = youTubeEntry.thumbnail
	res.append(MultiContentEntryPixmapAlphaTest(pos = (10, 5), size = (130, 97), png = png))

	return res


class YouTubeEntry():
	def __init__(self, priority, title, description, youTubeId, sourceFile):
		self.priority = priority
		self.title = title
		self.description = description
		self.youTubeId = youTubeId
		self.sourceFile = sourceFile
		self.thumbnail = None

	def setThumbNail(self, thumbNail):
		self.thumbnail = thumbNail


class YouTubeList(MenuList):
	def __init__(self, list, enableWrapAround = False):
		MenuList.__init__(self, list, enableWrapAround, eListboxPythonMultiContent)
		self.l.setFont(0, gFont("Regular", 18))
		self.l.setFont(1, gFont("Regular", 14))
		self.l.setItemHeight(105)


# This should be executed only once during an enigma2-session
config.plugins.youtubeplayer = ConfigSubsection()
config.plugins.youtubeplayer.serverprofile = ConfigText("", False)


class YouTubeListScreen(Screen, NumericalTextInput):
	skin = """
	<screen name="YouTubeListScreen" position="80,85" size="560,410" title="YouTube List" transparent="0">
		<widget name="list" position="5,10" size="550,350" scrollbarMode="showOnDemand" />

		<ePixmap name="key_red" position="40,365" zPosition="1" size="200,40" pixmap="skin_default/buttons/red-big.png" transparent="1" alphatest="on" />
		
		<ePixmap name="key_green" position="320,365" zPosition="1" size="200,40" pixmap="skin_default/buttons/green-big.png" transparent="1" alphatest="on" />
		
		<widget name="red" position="40,365" size="200,40" font="Regular;19" valign="center" halign="center" backgroundColor="background" transparent="1" zPosition="2" />
		
		<widget name="green" position="320,365" size="200,40" font="Regular;19" valign="center" halign="center" backgroundColor="background" transparent="1" zPosition="2" />
	</screen>"""
	
	def __init__(self, session):
		Screen.__init__(self, session)
		NumericalTextInput.__init__(self)

		self.session = session
		
		self.serverName = config.plugins.youtubeplayer.serverprofile.value
		
		self["red"] = Label(_("Select a VLC-Server"))

		self["green"] = Label(_("New YouTube search"))

		self.list = []
		self["list"] = YouTubeList(self.list)

		self.service = gdata.service.GDataService(None, None, "HOSTED_OR_GOOGLE", None, None, None, "gdata.youtube.com")
		
		self["actions"] = ActionMap(["WizardActions", "ColorActions"],
		{
			"ok": 		self.tryToPlay,
			"red": 		self.justSelectServer,
			"green":	self.search,
			"back": 	self.close
		})
		self.onFirstExecBegin.append(self.search)

# http://cacan.blog385.com/index.php/2008/05/09/youtube-high-quality-hacks/
# add the &fmt=6 onto the end:
#
# http://youtube.com/watch?v=CQzUsTFqtW0&fmt=6
#
# If the YouTube video just sits there loading then that
# is a sign that the video has not been converted to the
# higher resolution yet. To really see the difference you
# should view the video in full screen mode by clicking
# the button in the bottom-right corner of the player.
#
# Note: Alternatively you can add &fmt=18 and it will play
# the high-resolution version when available, otherwise it
# will play the regular version. Here?s a Greasemonkey
# script that will automatically add &fmt=18 onto the end
# of each YouTube URL.


	def search(self):
		self.help_window = self.session.instantiateDialog(NumericalTextInputHelpDialog, self)
		self.help_window.show()
		self.session.openWithCallback(self.doSearch, InputBox, "Enter Search Terms", "YouTube Search")

	def doSearch(self, searchTerms):
		print "[YTB] doSearch ", searchTerms
		if self.help_window:
			self.session.deleteDialog(self.help_window)
			self.help_window = None
		if searchTerms is not None:
			self.searchTerm = searchTerms
			self.setTitle("YouTube: " + searchTerms)
			self.getResults("/feeds/videos?vq=" + urllib.quote_plus(searchTerms) + "&orderby=relevance&max-results=20")
		else:
			self.close()

	def getResults(self, url):
		print "[YTB] getResults ", url
		self.list = []
		entries = self.service.Get(url).entry
		i = 0
		self.youTubeEntries = []
		for entry in entries:
			for element in entry.extension_elements:
				if element.tag == "group":
					break
			print "[YTB] name", entry.title.text
			sourcefile = "/tmp/thumbnail" + str(i) + ".jpg"
			i = i + 1
			youTubeId = entry.id.text.split("/").pop()
			description = element.FindChildren("description")[0].text
			youTubeEntry = YouTubeEntry(i, entry.title.text, description, youTubeId, sourcefile)
			thumbnailUrl = element.FindChildren("thumbnail")[0].attributes['url']
			downloadPage(thumbnailUrl, sourcefile).addCallback(self.fetchFinished, youTubeEntry).addErrback(self.fetchFailed, youTubeEntry)
		self["list"].l.setList(self.list)
		print "[YTB] getResults end"

	def fetchFailed(self, string, youTubeEntry):
		if os.path.exists(youTubeEntry.sourceFile):
			os.remove(youTubeEntry.sourceFile)
		self.list.append(YouTubeEntryComponent(youTubeEntry))
		self.list.sort(cmp = lambda x, y : cmp(x[0].priority, y[0].priority))
		self["list"].l.invalidate()

	def fetchFinished(self, string, youTubeEntry):
		if os.path.exists(youTubeEntry.sourceFile):
			thumbNail = LoadPixmap(youTubeEntry.sourceFile)
			youTubeEntry.setThumbNail(thumbNail)
			os.remove(youTubeEntry.sourceFile)
		self.list.append(YouTubeEntryComponent(youTubeEntry))
		self.list.sort(cmp = lambda x, y : cmp(x[0].priority, y[0].priority))
		self["list"].l.invalidate()

	def justSelectServer(self):
		self.selectServer(self.serverSelectedCB)
		
	def selectServer(self, callback):
		self.session.openWithCallback(callback, VlcServerListScreen, False)

	def serverSelectedCB(self, selectedServer):
		if selectedServer is not None:
			config.plugins.youtubeplayer.serverprofile.value = selectedServer.getName()
			config.plugins.youtubeplayer.serverprofile.save()

	def selectAndPlayCB(self, selectedServer):
		if selectedServer is not None:
			self.serverSelectedCB(selectedServer)
			self.play(selectedServer)

	def tryToPlay(self):
		currentServer = vlcServerConfig.getServerByName(config.plugins.youtubeplayer.serverprofile.value)
		if currentServer is not None:
			self.play(currentServer)
		else:
			self.selectServer(self.selectAndPlayCB)

	def play(self, currentServer):
		print "[YTB] Play()"
		youTubeEntry = self["list"].getCurrent()[0]
		youTubeId = youTubeEntry.youTubeId
		conn = httplib.HTTPConnection("www.youtube.com")
		conn.request("GET", "/v/" + urllib.quote(youTubeId))
		response = conn.getresponse()
		conn.close()
		if response.status == 303:
			location = response.getheader("location")
			mrl = "http://www.youtube.com/get_video?video_id=" + urllib.quote(youTubeId) + "&t=" + urllib.quote(re.match (".*[?&]t=([^&]+)", location).groups ()[0]) + "&fmt=18"
			print "[YTB] Playing ", mrl
			currentServer.play(self, mrl, youTubeEntry.title)
		else:
			print "[YTB] No valid flv-mrl found"
